@@ -429,9 +429,10 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		return tags.Contains (tag);
 	}
 
-	private static FSpot.Xmp.XmpFile CreateXmp (FSpot.IBrowsableItem item)
+	private static FSpot.Xmp.XmpFile UpdateXmp (FSpot.IBrowsableItem item, FSpot.Xmp.XmpFile xmp)
 	{
-		FSpot.Xmp.XmpFile xmp = new FSpot.Xmp.XmpFile ();
+		if (xmp == null) 
+			xmp = new FSpot.Xmp.XmpFile ();
 
 		Tag [] tags = item.Tags;
 		string [] names = new string [tags.Length];
@@ -439,7 +440,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		for (int i = 0; i < tags.Length; i++)
 			names [i] = tags [i].Name;
 		
-		FSpot.MetadataStore.Add (xmp, "dc:subject", "rdf:Bag", names);
+		xmp.Store.Update ("dc:subject", "rdf:Bag", names);
 		xmp.Dump ();
 
 		return xmp;
@@ -455,10 +456,18 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 			
 			jimg.SetDescription (this.Description);
 			jimg.SetDateTimeOriginal (this.Time);
-			FSpot.Xmp.XmpFile xmp = CreateXmp (this);
-			jimg.SetXmp (xmp);
+			jimg.SetXmp (UpdateXmp (this, jimg.Header.GetXmp ()));
 
 			jimg.SaveMetaData (path);
+		} else if (img is FSpot.Png.PngFile) {
+			FSpot.Png.PngFile png = img as FSpot.Png.PngFile;
+			
+			if (img.Description != this.Description)
+				png.SetDescription (this.Description);
+			
+			png.SetXmp (UpdateXmp (this, png.GetXmp ()));
+
+			png.Save (path);
 		}
 	}
 
@@ -492,7 +501,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 	}
 
 	// Constructor
-	public Photo (uint id, uint unix_time, string directory_path, string name)
+	public Photo (uint id, long unix_time, string directory_path, string name)
 		: base (id)
 	{
 		time = DbUtils.DateTimeFromUnixTime (unix_time);
@@ -507,7 +516,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		AddVersionUnsafely (OriginalVersionId, Mono.Posix.Catalog.GetString ("Original"));
 	}
 
-	public Photo (uint id, uint unix_time, string path)
+	public Photo (uint id, long unix_time, string path)
 		: this (id, unix_time,
 
 			System.IO.Path.GetDirectoryName (path),
@@ -646,7 +655,7 @@ public class PhotoStore : DbStore {
 	public Photo Create (string newPath, string origPath, out Pixbuf thumbnail)
 	{
 		FSpot.ImageFile img = FSpot.ImageFile.Create (origPath);
-		uint unix_time = DbUtils.UnixTimeFromDateTime (img.Date);
+		long unix_time = DbUtils.UnixTimeFromDateTime (img.Date);
 		string description = img.Description != null ? img.Description : "";
 		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
@@ -860,7 +869,7 @@ public class PhotoStore : DbStore {
 
 		if (reader.Read ()) {
 			photo = new Photo (id,
-					   Convert.ToUInt32 (reader [0]),
+					   Convert.ToInt64 (reader [0]),
 					   reader [1].ToString (),
 					   reader [2].ToString ());
 
@@ -908,7 +917,7 @@ public class PhotoStore : DbStore {
 
 		if (reader.Read ()) {
 			photo = new Photo (Convert.ToUInt32 (reader [0]),
-					   Convert.ToUInt32 (reader [1]),
+					   Convert.ToInt64 (reader [1]),
 					   directory_path,
 					   filename);
 
@@ -1015,10 +1024,12 @@ public class PhotoStore : DbStore {
 		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
 		command.CommandText = String.Format ("UPDATE photos SET description = '{0}',     " +
-						     "                  default_version_id = {1} " +
-						     "              WHERE id = {2}",
+						     "                  default_version_id = {1}, " +
+						     "                  time = {2} " +
+						     "              WHERE id = {3}",
 						     SqlString (photo.Description),
 						     photo.DefaultVersionId,
+						     DbUtils.UnixTimeFromDateTime (photo.Time),
 						     photo.Id);
 		command.ExecuteNonQuery ();
 		command.Dispose ();
@@ -1113,7 +1124,7 @@ public class PhotoStore : DbStore {
 
 			if (photo == null) {
 				photo = new Photo (id,
-						   Convert.ToUInt32 (reader [1]),
+						   Convert.ToInt64 (reader [1]),
 						   reader [2].ToString (),
 						   reader [3].ToString ());
 				
