@@ -11,6 +11,9 @@ namespace FSpot {
 
 		[Glade.Widget] Gtk.CheckMenuItem side_pane_item;
 		[Glade.Widget] Gtk.CheckMenuItem toolbar_item;
+		
+		[Glade.Widget] Gtk.MenuItem zoom_in;
+		[Glade.Widget] Gtk.MenuItem zoom_out;
 
 		[Glade.Widget] Gtk.Image near_image;
 		[Glade.Widget] Gtk.Image far_image;
@@ -22,7 +25,10 @@ namespace FSpot {
 		PhotoImageView image_view;
 		IconView directory_view;
 		string path;
-
+		private 
+		
+		InfoDialog metadata_dialog;
+		
 		DirectoryCollection collection;
 		
 		FSpot.Delay slide_delay;
@@ -39,6 +45,9 @@ namespace FSpot {
 			xml = new Glade.XML (null, "f-spot.glade", glade_name, "f-spot");
 			xml.Autoconnect (this);
 			window = (Gtk.Window) xml.GetWidget (glade_name);
+		
+			LoadPreference (Preferences.VIEWER_WIDTH);
+			LoadPreference (Preferences.VIEWER_MAXIMIZED);
 
 			Gtk.Toolbar toolbar = new Gtk.Toolbar ();
 			toolbar_hbox.PackStart (toolbar);
@@ -63,14 +72,18 @@ namespace FSpot {
 			Window.ShowAll ();
 
 			zoom_scale.ValueChanged += HandleZoomScaleValueChanged;
+		
+			LoadPreference (Preferences.VIEWER_SHOW_TOOLBAR);
 			
-			ShowToolbar = true;
 			ShowSidebar = collection.Count > 1;
 
 			near_image.SetFromStock ("f-spot-stock_near", Gtk.IconSize.SmallToolbar);
 			far_image.SetFromStock ("f-spot-stock_far", Gtk.IconSize.SmallToolbar);
 
 			slide_delay = new FSpot.Delay (new GLib.IdleHandler (SlideShow));
+
+			Preferences.SettingChanged += OnPreferencesChanged;
+			window.DeleteEvent += HandleDeleteEvent;
 			
 			if (collection.Count > 0)
 				directory_view.Selection.Add (0);
@@ -119,6 +132,8 @@ namespace FSpot {
 			if (selection.Count > 0) {
 				image_view.Item.Index = ((IconView.SelectionCollection)selection).Ids[0];
 				zoom_scale.Value = image_view.NormalizedZoom;
+				if (metadata_dialog != null)
+					metadata_dialog.InfoDisplay.Photo = image_view.Item.Current;
 			}
 		}
 
@@ -142,7 +157,26 @@ namespace FSpot {
 			this.Window.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.Watch);
 			slide_delay.Start ();
 		}
+	
+		private void HandleViewMetadata (object sender, System.EventArgs args)
+		{
+			if (metadata_dialog != null) {
+				metadata_dialog.Present ();
+				return;
+			}
+			
+			metadata_dialog = new InfoDialog (window);
+			metadata_dialog.InfoDisplay.Photo = image_view.Item.Current;
+			
+			metadata_dialog.ShowAll ();
+			metadata_dialog.Destroyed += HandleMetadataDestroyed;
+		}
 		
+		private void HandleMetadataDestroyed (object sender, System.EventArgs args)
+		{
+			metadata_dialog = null;
+		}
+	
 		private bool SlideShow ()
 		{
 			IBrowsableItem [] items = new IBrowsableItem [collection.Count];
@@ -194,13 +228,42 @@ namespace FSpot {
 		private void HandleZoomChanged (object sender, System.EventArgs args)
 		{
 			zoom_scale.Value = image_view.NormalizedZoom;
+
+			// FIXME something is broken here
+			//zoom_in.Sensitive = (zoom_scale.Value != 1.0);
+			//zoom_out.Sensitive = (zoom_scale.Value != 0.0);
+		}
+	
+		void HandleDeleteEvent (object sender, DeleteEventArgs args)
+		{
+			SavePreferences ();
+			this.Window.Destroy ();
+			args.RetVal = true;
 		}
 
 		private void HandleFileClose (object sender, System.EventArgs args)
 		{
+			SavePreferences ();
 			this.Window.Destroy ();
 		}
 
+
+		private void SavePreferences  ()
+		{
+			int width, height;
+			window.GetSize (out width, out height);
+		
+			bool maximized = ((window.GdkWindow.State & Gdk.WindowState.Maximized) > 0);
+			Preferences.Set (Preferences.VIEWER_MAXIMIZED, maximized);
+		
+			if (!maximized) {
+				Preferences.Set (Preferences.VIEWER_WIDTH,	width);
+				Preferences.Set (Preferences.VIEWER_HEIGHT,	height);
+			}
+		
+			Preferences.Set (Preferences.VIEWER_SHOW_TOOLBAR,	toolbar_hbox.Visible);
+		}
+
 		private void HandleFileOpen (object sender, System.EventArgs args)
 		{
 			string open = null;
@@ -223,6 +286,43 @@ namespace FSpot {
 			file_selector.Destroy ();
 		}
 
+		void OnPreferencesChanged (object sender, GConf.NotifyEventArgs args)
+		{
+			LoadPreference (args.Key);
+		}
+
+		void LoadPreference (String key)
+		{
+			object val = Preferences.Get (key);
+
+			if (val == null)
+				return;
+			
+			switch (key) {
+			case Preferences.VIEWER_MAXIMIZED:
+				if ((bool) val)
+					window.Maximize ();
+				else
+					window.Unmaximize ();
+				break;
+
+			case Preferences.VIEWER_WIDTH:
+			case Preferences.VIEWER_HEIGHT:
+				window.SetDefaultSize((int) Preferences.Get(Preferences.VIEWER_WIDTH),
+						(int) Preferences.Get(Preferences.VIEWER_HEIGHT));
+
+				window.ReshowWithInitialSize();
+				break;
+			
+			case Preferences.VIEWER_SHOW_TOOLBAR:
+				if (toolbar_item.Active != (bool) val)
+					toolbar_item.Active = (bool) val;
+
+				toolbar_hbox.Visible = (bool) val;
+				break;
+			}
+		}
+
 		public Gtk.Window Window {
 			get { 
 				return window;
