@@ -527,6 +527,7 @@ namespace FSpot.Tiff {
 		Custom = 1
 	}
 
+
 	public enum SceneType {
 		DirectlyPhotographed = 1
 	}
@@ -762,7 +763,7 @@ namespace FSpot.Tiff {
 			ushort marker = BitConverter.ToUInt16 (data, 2, endian == Endian.Little);
 			switch (marker) {
 			case 42:
-				System.Console.WriteLine ("Found Standard Tiff Marker {0}", marker);
+				//System.Console.WriteLine ("Found Standard Tiff Marker {0}", marker);
 				break;
 			case 0x4f52:
 				System.Console.WriteLine ("Found Olympus Tiff Marker {0}", marker.ToString ("x"));
@@ -861,8 +862,6 @@ namespace FSpot.Tiff {
 						System.Console.WriteLine (String.Format ("error parsing {0}\n{1}", e.ValueAsString[0], ex));
 					}
 					break;
-					//case TagId.Flash:
-					
 					//case TagId.SpatialFrequencyResponse
 				case TagId.ExifCFAPattern:
 					CFAPattern pattern = new CFAPattern (e.RawData, e.IsLittle);
@@ -1649,6 +1648,8 @@ namespace FSpot.Tiff {
 					case TagId.FileSource:
 					case TagId.SceneType:
 						return ArrayToString (this.RawData);
+					case TagId.ComponentsConfiguration:
+						return ArrayToString (ValueAsLong);
 					default:
 						System.Console.WriteLine ("Cannot convert type \"{0}\" to string", Id);
 						break;
@@ -1681,6 +1682,7 @@ namespace FSpot.Tiff {
 					case EntryType.Short:
 						data [i] = BitConverter.ToUInt16 (raw_data, i * GetTypeSize (), endian == Endian.Little);
 						break;
+					case EntryType.Undefined:
 					case EntryType.Byte:
 						data [i] = raw_data [i];
 						break;
@@ -1773,7 +1775,9 @@ namespace FSpot.Tiff {
 					this.Header = new Header (input);
 				}
 
-				//Header.Dump ("");
+#if DEBUG_LOADER
+				Header.Dump (this.ToSring () + ":");
+#endif
 			} catch (System.Exception e) {
 				System.Console.WriteLine (e.ToString ());
 			}
@@ -1786,8 +1790,18 @@ namespace FSpot.Tiff {
 
 		public override System.DateTime Date {
 			get {
-				DirectoryEntry e = this.Header.Directory.Lookup (TagId.DateTime);
-				
+				SubdirectoryEntry sub = (SubdirectoryEntry) this.Header.Directory.Lookup (TagId.ExifIfdPointer);
+				DirectoryEntry e;
+
+				if (sub != null) {
+					e = sub.Directory [0].Lookup (TagId.DateTimeOriginal);
+					
+					if (e != null)
+						return DirectoryEntry.DateTimeFromString (e.StringValue);
+				}
+
+				e = this.Header.Directory.Lookup (TagId.DateTime);
+
 				if (e != null)
 					return DirectoryEntry.DateTimeFromString (e.StringValue);
 				else
@@ -1847,17 +1861,85 @@ namespace FSpot.Tiff {
 	}
 
 	public class DngFile : TiffFile {
-		public DngFile (string path) : base (path) {}
-
-		public override Gdk.Pixbuf Load (int width, int height)
+		public DngFile (string path) : base (path) 
 		{
-			return PixbufUtils.ScaleToMaxSize (this.Load (), width, height);
+		}
+
+		public override System.IO.Stream PixbufStream ()
+		{
+			try {
+				SubdirectoryEntry sub = (SubdirectoryEntry) Header.Directory.Lookup (TagId.SubIFDs);
+				ImageDirectory directory = sub.Directory [sub.Directory.Length - 1];
+
+				uint offset = directory.Lookup (TagId.StripOffsets).ValueAsLong [0];
+				System.IO.Stream file = System.IO.File.OpenRead (this.path);
+				file.Position = offset;
+				return file;
+			} catch {
+				return DCRawFile.RawPixbufStream (path);
+			}
 		}
 
 		public override Gdk.Pixbuf Load ()
 		{
 			return DCRawFile.Load (this.path, null);
 		}
+
+		public override void Select (SemWeb.StatementSink sink)
+		{
+			
+			/* this is just a sanity pass, if the first ifd is not a subfile use the normal
+			 * tiff path 
+			 */
+			DirectoryEntry e = Header.Directory.Lookup (TagId.NewSubfileType);
+			if (e == null) {
+				base.Select (sink);
+				return;
+			}
+
+			/*
+			 * Even though Ifd0 doesn't have the full resolution image
+			 * it would have the XMP data so we look for it
+			 */
+			e = Header.Directory.Lookup (TagId.XMP);
+			if (e != null) {
+				System.IO.Stream xmpstream = new System.IO.MemoryStream (e.RawData);
+				FSpot.Xmp.XmpFile xmp = new FSpot.Xmp.XmpFile (xmpstream);
+				xmp.Select (sink);
+			}
+
+			/* 
+			 * Ifd0 will also have the exif directory
+			 */
+			ImageDirectory dir = Header.Directory;
+			SubdirectoryEntry sub = (SubdirectoryEntry) dir.Lookup (TagId.ExifIfdPointer);
+			if (sub != null)
+				Header.SelectDirectory (sub.Directory [0], sink);
+			
+			/*
+			 * now we lookup subifd0 (we should probably scan the newsubfile types here)
+			 * and load the metadata we are interested in from it.
+			 */
+			sub = (SubdirectoryEntry) Header.Directory.Lookup (TagId.SubIFDs);	
+
+			int i = 0;
+			do {
+				uint dirtype = e.ValueAsLong [0];
+				if (dirtype == 0) {
+					Header.SelectDirectory (dir, sink);
+					break;
+				}
+					
+				if (sub == null)
+					break;
+
+				dir = sub.Directory [i];
+				e = dir.Lookup (TagId.NewSubfileType);
+				i++;
+			} while (i < sub.Directory.Length);
+
+			
+		}
 	}	
 	
 	public class NefFile : TiffFile, IThumbnailContainer {
@@ -1911,7 +1993,7 @@ namespace FSpot.Tiff {
 				ImageDirectory jpeg_directory = sub.Directory [0];
 				return LookupJpegSubstream (jpeg_directory);
 			} catch (System.Exception e) {
-				return null;
+				return DCRawFile.RawPixbufStream (path);
 			}
 		}
 		
@@ -1935,11 +2017,6 @@ namespace FSpot.Tiff {
 			
 			return TransformAndDispose (pixbuf);
 		}
-
-		public override Gdk.Pixbuf Load (int width, int height)
-		{
-			return PixbufUtils.ScaleToMaxSize (this.Load (), width, height);
-		}
 	}
 		
 
@@ -1947,9 +2024,15 @@ namespace FSpot.Tiff {
 
 		public Cr2File (string path) : base (path) 
 		{
-			//this.Header.Dump ("loading");
 		}
-		
+
+		/*
+		public override PixbufOrientation GetOrientation ()
+		{
+			return PixbufOrientation.TopLeft;
+		}
+		*/
+
 		public Gdk.Pixbuf GetEmbeddedThumbnail ()
 		{
 			ImageDirectory directory;
@@ -1957,21 +2040,22 @@ namespace FSpot.Tiff {
 			return TransformAndDispose (LoadJpegInterchangeFormat (directory));
 		}
 
+
+		public override System.IO.Stream PixbufStream ()
+		{
+			uint offset = Header.Directory.Lookup (TagId.StripOffsets).ValueAsLong [0];
+			System.IO.Stream file = System.IO.File.OpenRead (this.path);
+			file.Position = offset;
+			return file;
+			//return LookupJpegSubstream (Header.Directory.NextDirectory);
+			//return DCRawFile.RawPixbufStream (path);
+		}
+
 		public override Gdk.Pixbuf Load ()
 		{
-#if false
-			return GetEmbeddedThumbnail ();
-#else
 			return DCRawFile.Load (this.Path, null);
-#endif
 		}
 
-		public override Gdk.Pixbuf Load (int width, int height)
-		{
-			return PixbufUtils.ScaleToMaxSize (this.Load (), width, height);
-		}
-
-
 		public override System.DateTime Date
 		{
 			get {
